/* Copyright (C) 2004,2005  The DESIROS Team
    desiros.dev@gmail.com

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.
   
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
   USA. 
 */
#include <types.h>
#include <kerrno.h>
#include <cpu_context.h>



inline int get_irq(cpu_state_t *user_ctxt,
			       /* out */unsigned int *irq)
{
  *irq = user_ctxt->int_no & 0xFF;

  return 0 ;
}


inline int syscall_get3args(cpu_state_t *user_ctxt,
			       /* out */unsigned int *arg1,
			       /* out */unsigned int *arg2,
			       /* out */unsigned int *arg3)
{
  *arg1 = user_ctxt->ebx;
  *arg2 = user_ctxt->ecx;
  *arg3 = user_ctxt->edx; 
  return 0 ;
}

int syscall_get1arg(cpu_state_t *user_ctxt,
			      /* out */unsigned int *arg1)
{
  unsigned int unused;
  return syscall_get3args(user_ctxt, arg1, & unused, & unused);
}


int syscall_get2args(cpu_state_t *user_ctxt,
			       /* out */unsigned int *arg1,
			       /* out */unsigned int *arg2)
{
  unsigned int unused;
  return syscall_get3args(user_ctxt, arg1, arg2, & unused);
}

int syscall_get4args(cpu_state_t *user_ctxt,
			       /* out */unsigned int *arg1,
			       /* out */unsigned int *arg2,
			       /* out */unsigned int *arg3,
			       /* out */unsigned int *arg4)
{
  __u32 uaddr_other_args;
  __u32 other_args[2];
  int    retval;

  /* Retrieve the 3 arguments. The last one is an array containing the
     remaining arguments */
  retval = syscall_get3args(user_ctxt, arg1, arg2,
				(unsigned int *)& uaddr_other_args);
  if (OK != retval)
    return retval;
  
   /* Copy the array containing the remaining arguments from user
     space */
  memcpy((unsigned int)other_args,
				(unsigned int)uaddr_other_args,
				sizeof(other_args));

  *arg3 = other_args[0];
  *arg4 = other_args[1];
  return OK;
}


int syscall_get7args(cpu_state_t *user_ctxt,
			       /* out */unsigned int *arg1,
			       /* out */unsigned int *arg2,
			       /* out */unsigned int *arg3,
			       /* out */unsigned int *arg4,
			       /* out */unsigned int *arg5,
			       /* out */unsigned int *arg6,
			       /* out */unsigned int *arg7)
{
  __u32  uaddr_other_args;
  unsigned int other_args[5];
  int    retval;

  /* Retrieve the 3 arguments. The last one is an array containing the
     remaining arguments */
  retval = syscall_get3args(user_ctxt, arg1, arg2,
				(unsigned int *)& uaddr_other_args);
  if (OK != retval)
    return retval;
  
  /* Copy the array containing the remaining arguments from user
     space */
  memcpy((unsigned int)other_args,
				(unsigned int)uaddr_other_args,
				sizeof(other_args));


  *arg3 = other_args[0];
  *arg4 = other_args[1];
  *arg5 = other_args[2];
  *arg6 = other_args[3];
  *arg7 = other_args[4];
  return OK;
}

