/* Copyright (C) 2004,2005  The DESIROS Team
    desiros.dev@gmail.com

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.
   
   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
   
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
   USA. 
 */


#include <types.h>
#include <idt.h>
#include <klibc.h>
#include <cpu_context.h>
#include <interrupt.h>
#include <debug.h>

/*
 * An IRQ handler pointer. */
typedef struct irq_desc
{
	void (*handler)(cpu_state_t* );
} irq_desc_t;


void unhandled_interrupt(struct cpu_state *user_ctxt);

/* A table for hardware interrupt handlers only. */
irq_desc_t irq_handler[ IRQ_NR ] __attribute__ ((aligned(16)));



extern void  _irq_00(), _irq_01(),_irq_02(), _irq_03(), _irq_04(), _irq_05(), _irq_06(), _irq_07();
extern void _irq_08(), _irq_09(), _irq_0A(), _irq_0B(), _irq_0C(), _irq_0D(), _irq_0E(), _irq_0F();
extern void _irq_unhand();


extern void _exc_00(), _exc_01(), _exc_02(), _exc_03(), _exc_04(), _exc_05(), _exc_06(), _exc_07(), _exc_08();
extern void _exc_09(), _exc_0A(), _exc_0B(), _exc_0C(), _exc_0D(), _exc_0E(), _exc_0F(),_exc_10(),_exc_11(), _exc_12() ;
extern void _exc_unhand();
extern void _asm_syscalls();

 /*
  * 'init_idt_desc' initializes a segment descriptor located idt.
  * param i The index of the entry.
  * The 'type' argument must have for INTGATE value TRAPGATE
  * or TASKGATE.
  */
void init_idt_desc(__u16 select, __u32 offset, __u16 type, __u8 i)
{
	kidt[i].offset0_15 = (offset & 0xffff);
	kidt[i].select = select;
	kidt[i].type = type;
	kidt[i].offset16_31 = (offset & 0xffff0000) >> 16;
	return;
}


void install_trap_handler( __u32 irq, void *handler )
{
        if ( irq > IRQ_NR )
		return;

	irq_handler[ irq ].handler = handler;

	return;
}

//! Setup a trap gate.
static void set_trap_gate( unsigned int n, void *addr )
{
	init_idt_desc( 0x08, (__u32)addr,TRAPGATE, n );
}



void default_handler(cpu_state_t *user_ctxt)
{

        if( user_ctxt->int_no  < IRQ_NR )
	{
	// Call the correct interrupt handler.
       irq_handler[ user_ctxt->int_no ].handler(user_ctxt);
        } else {
         debug("Unexpected IRQ number %03u!\n", user_ctxt->int_no);

        }
}


/*
 * This function initializes the IDT after the kernel is loaded
 * in memory 
 */
void init_idt(void)
{
	int i;

// Setup the IDT entries for exceptions.

	set_trap_gate( 0x00, &_exc_00 ); // divide error.
	set_trap_gate( 0x01, &_exc_01 ); // debug.
	set_trap_gate( 0x02, &_exc_02 ); // nmi.
	set_trap_gate( 0x03, &_exc_03 ); // int3.
	set_trap_gate( 0x04, &_exc_04 ); // overflow.
	set_trap_gate( 0x05, &_exc_05 ); // bounds.
	set_trap_gate( 0x06, &_exc_06 ); // invalid operation.
	set_trap_gate( 0x07, &_exc_07 ); // device not available.
	set_trap_gate( 0x08, &_exc_08 ); // double fault.
	set_trap_gate( 0x09, &_exc_09 ); // coprocessor segment overrun.
	set_trap_gate( 0x0a, &_exc_0A ); // invalid tss.
	set_trap_gate( 0x0b, &_exc_0B ); // segment not present.
	set_trap_gate( 0x0c, &_exc_0C ); // stack fault.
	set_trap_gate( 0x0d, &_exc_0D ); // general protection.
	set_trap_gate( 0x0e, &_exc_0E ); // page fault.
	set_trap_gate( 0x0f, &_exc_0F ); // spurious interrupt.
	set_trap_gate( 0x10, &_exc_10);
	set_trap_gate( 0x11, &_exc_11);
	set_trap_gate( 0x12, &_exc_12);

    // Initialize interrupt handlers 0x20..0xFF.
	for( i = 0x20; i <= 0xFF; i++ )
		set_trap_gate( i, &_irq_unhand );

	// Setup the IDT entries for IRQs.
	set_trap_gate( 0x20, &_irq_00 );
	set_trap_gate( 0x21, &_irq_01 );
	set_trap_gate( 0x22, &_irq_02 );
	set_trap_gate( 0x23, &_irq_03 );
	set_trap_gate( 0x24, &_irq_04 );
	set_trap_gate( 0x25, &_irq_05 );
	set_trap_gate( 0x26, &_irq_06 );
	set_trap_gate( 0x27, &_irq_07 );
	set_trap_gate( 0x28, &_irq_08 );
	set_trap_gate( 0x29, &_irq_09 );
	set_trap_gate( 0x2a, &_irq_0A );
	set_trap_gate( 0x2b, &_irq_0B );
	set_trap_gate( 0x2c, &_irq_0C );
	set_trap_gate( 0x2d, &_irq_0D );
	set_trap_gate( 0x2e, &_irq_0E );
	set_trap_gate( 0x2f, &_irq_0F );
        set_trap_gate( 0x80, &_asm_syscalls );



	/* Initialization system default descriptor IDTR */
	kidtr.limite = IDTSIZE * 8;
	kidtr.base = IDTBASE;

	/* Copy of the IDT's address */
	memmove((char *) kidtr.base, (char *) kidt, kidtr.limite);

	/* Load register IDTR */
	asm("lidtl (kidtr)");
}
