
#ifndef _CPU_CONTEXT_H_
#define _CPU_CONTEXT_H_

/**
 * Opaque structure storing the CPU context of an inactive kernel or
 * user thread, as saved by the low level primitives below or by the
 * interrupt/exception handlers.
 */

typedef struct cpu_state {

  /* Segment registers */
  __u32 gs;
  __u32 fs;
  __u32 es;
  __u32 ds;

/* General registers, accessed with pushal/popal */
  __u32 edi;
  __u32 esi; 
  __u32 ebp; 
  __u32 esp;   /* points just below eax */
  __u32 ebx;  
  __u32 edx; 
  __u32 ecx; 
  __u32 eax;

    unsigned int int_no, err_code;
    unsigned int eip, cs, eflags, useresp, ss;

} __attribute__((packed)) cpu_state_t;


typedef struct cpu_state cpu_state_t ;

inline int get_irq(cpu_state_t *user_ctxt,
			       /* out */unsigned int *irq);

inline int syscall_get3args(cpu_state_t *user_ctxt,
			       unsigned int *arg1,
			       unsigned int *arg2,
			       unsigned int *arg3);

int syscall_get1arg(cpu_state_t *user_ctxt,
			      unsigned int *arg1);

int syscall_get2args(cpu_state_t *user_ctxt,
			       unsigned int *arg1,
			       unsigned int *arg2);

#endif


