/*!	\file include/dma.h
 *	\brief DMA (Direct Memory Access) driver header.
 */

#ifndef DMA_H
#define DMA_H

//! DMA memory area start address.
#define DMA_MEMORY_START	0x3000

//! DMA memory area end address.
#define DMA_MEMORY_END		dma_memory_end


//! The size of the DMA memory area.
#define DMA_MEMORY_DIM (DMA_MEMORY_END - DMA_MEMORY_START)

//! The DMA-memory frame is marked as free.
#define DMA_FREE_FRAME	0

//! The DMA-memory frame is marked as busy.
#define DMA_BUSY_FRAME	1

//! Normalize the physical address to the dma base address.
#define DMA_FRAME_ADDR( i ) \
	(((i) < DMA_MEMORY_START) ? 0 : ((size_t)(i) - DMA_MEMORY_START))

//! A structure used to store hardware definition of DMA channels.
typedef struct dma_channel
{
	__u8 page;     //!< Page register.
	__u8 offset;   //!< Offset register.
	__u8 length;   //!< Length register.
} dma_channel_t;

extern size_t dma_memory_end;

// --- Prototypes ----------------------------------------------------- //

void dma_setup( size_t phys_mem );

void lock_dma_controller();
void unlock_dma_controller();

void dma_xfer(unsigned channel, __u32 physaddr, size_t length, bool read);
void *dma_phys_alloc(size_t len);
void dma_phys_free(size_t dma_start, size_t len);
size_t dma_get_free_mem();

void dma_setup_memory_end( size_t phys_mem );
void dma_alloc_init();

#endif

