# Copyright (C) 2004,2005  The DESIROS Team
#      desiros.dev@gmail.com
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
# USA.

.global _irq_00, _exc_0E,_asm_syscalls,_go,reschedule,_irq_01

.text

; //! Associates the name with the number
#define INTERRUPT(name, irq) \
	.global  _##name ; \
_##name: \
	pushl	##irq ; \
	jmp	_asm_default_int ; \

#define EXCEPTION(name, irq) \
	.global  _##name ; \
_##name: \
	pushl	##irq ; \
	jmp	_asm_execpt_int ; \


; // --- Interrupt Service Routines Declaration ------------------------//

INTERRUPT(irq_02, $0x02)	; //!< PIC Slave
INTERRUPT(irq_03, $0x03)	; //!< RS232-(COM2) or Available
INTERRUPT(irq_04, $0x04)	; //!< RS232-(COM1)
INTERRUPT(irq_05, $0x05)	; //!< Available
INTERRUPT(irq_06, $0x06)	; //!< Floppy Disk
INTERRUPT(irq_07, $0x07)	; //!< LPT1
INTERRUPT(irq_08, $0x08)	; //!< CMOS Real Time Clock
INTERRUPT(irq_09, $0x09)	; //!< PIC Master
INTERRUPT(irq_0A, $0x0A)	; //!< Available
INTERRUPT(irq_0B, $0x0B)	; //!< Available
INTERRUPT(irq_0C, $0x0C)	; //!< Mouse PS/2 or Available
INTERRUPT(irq_0D, $0x0D)	; //!< Numeric Coprocessor
INTERRUPT(irq_0E, $0x0E)	; //!< IDE Channel 0 (Master)
INTERRUPT(irq_0F, $0x0F)	; //!< IDE Channel 1 (Slave)


INTERRUPT(irq_unhand, $0xFF)	; //!< Unhandled interrupt


        EXCEPTION( exc_00, $0x00); // divide error.
	EXCEPTION( exc_01, $0x01); // debug.
	EXCEPTION( exc_02, $0x02); // nmi.
	EXCEPTION( exc_03, $0x03); // int3.
	EXCEPTION( exc_04, $0x04); // overflow.
	EXCEPTION( exc_05, $0x05); // bounds.
	EXCEPTION( exc_06, $0x06); // invalid operation.
	EXCEPTION( exc_07, $0x07); // device not available.
	EXCEPTION( exc_08, $0x08); // double fault.
	EXCEPTION( exc_09, $0x09); // coprocessor segment overrun.
	EXCEPTION( exc_0A, $0x0A); // invalid tss.
	EXCEPTION( exc_0B, $0x0B); // segment not present.
	EXCEPTION( exc_0C, $0x0C); // stack fault.
	EXCEPTION( exc_0D, $0x0D); // general protection.
	EXCEPTION( exc_0F, $0x0F); // spurious interrupt.
	EXCEPTION( exc_10, $0x10);
	EXCEPTION( exc_11, $0x11);
	EXCEPTION( exc_12, $0x12);


.macro	SAVE_REGS 

  
  pushal
  pushl %ds
  pushl %es
  pushl %fs
  pushl %gs
 
  movw $0x10,%bx
  movw %bx,%ds
  movw %bx,%es
  movw %bx,%fs
  movw %bx,%gs


.endm

.macro	RESTORE_REGS 

  /* Restore the user context */
  
  popl %gs
  popl %fs
  popl %es
  popl %ds
  popal
.endm



_asm_default_int:
	SAVE_REGS
        pushl	%esp
	call default_handler
	movb $0x20,%al
	outb %al,$0x20
	RESTORE_REGS
	iret

_asm_execpt_int:
	SAVE_REGS
        pushl	%esp
	call isr_except_int
	movb $0x20,%al
	outb %al,$0x20
	RESTORE_REGS
	iret

// Clock
_irq_00:
	SAVE_REGS
	call isr_clock_int
	movb $0x20,%al
	outb %al,$0x20
	RESTORE_REGS
	iret

// Keyboard
_irq_01:
	SAVE_REGS
	call isr_kbd
	mov $0x20,%al
	outb %al,$0x20
	RESTORE_REGS
	iret
// Page Fault
_exc_0E :
	SAVE_REGS
	call isr_page_fault
	RESTORE_REGS
        add  $4,%esp 
	iret

_asm_syscalls:
	SAVE_REGS
	pushl %esp /* user_ctxt */
	pushl %eax
	call do_syscalls
	addl $8, %esp
        /* store the do_syscall return value into interrupted context */
        movl %eax,44(%esp)
	RESTORE_REGS
	iret

reschedule:
	SAVE_REGS
        pushl %eax
	call do_reschedule
	RESTORE_REGS
	iret

