
#include <fd_types.h>
#include <debug.h>
#include <process.h>
#include <vfs.h>
#include <io.h>

int sys_write( __u32 fd, const void *buf, __u32 c) {
	struct process     *process = current ;
	int ret ;
	
	open_file_descriptor *ofd;

	if (process->fd[fd]) {
		ofd = process->fd[fd];
		
		if(ofd->f_ops->write == NULL) {
			debug("No \"write\" method for this device.");
			ret = -1;
		} else {
			ret = ofd->f_ops->write(ofd, (void*) buf, c);
		}
	} else {
		ret = -1;
	}

         return ret;
}

int sys_open( char *path , __u32 flags) {
	int i=0;
	int fd_id;
	struct process     *process = current ;
	
	// recherche d une place dans la file descriptor table du process
	while (process->fd[i] != NULL ) { 
		i++;
	}

           
	// ouverture du fichier
	if ((process->fd[i] = vfs_open(path, flags)) != NULL) {
		fd_id = i;
            

	} else {
		fd_id = -1;
	}

        return fd_id;

}

int sys_close(int fd_id)
{
	struct process     *process = current ;
	
	open_file_descriptor *ofd;

	if (fd_id >= 0 && fd_id < FOPEN_MAX && process->fd[fd_id]) {
		ofd = process->fd[fd_id];
	
		if(ofd->f_ops->close == NULL)
		 debug("No \"close\" method for this device.");
		else
		ofd->f_ops->close(ofd);          
		vfs_close(ofd);
		process->fd[fd_id] = NULL;

                return 0 ;
                
	}

         return -1 ;
}

int sys_read( __u32 fd,void *buf, __u32 c) {

	struct process     *process = current ;
	int ret ;
	
	open_file_descriptor *ofd;



	if (process->fd[fd]) {
		ofd = process->fd[fd];
		
		if(ofd->f_ops->read == NULL) {
			debug("No \"read\" method for this device.");
			ret = -1;
		} else {

			ret = ofd->f_ops->read(ofd, (void*) buf, c);
		}
	} else {
		ret = -1;
	}

         return ret;
}

int sys_seek( __u32 fd, long *offset, int whence) {
	struct process     *process = current ;
	int ret ;
	
	open_file_descriptor *ofd;

	if (process->fd[fd]) 
		ofd = process->fd[fd];
		
		if(ofd->f_ops->seek == NULL) {
			debug("No \"seek\" method for this device.");
			*offset = -1;
		} else {
			ofd->f_ops->seek(ofd, *offset, whence);
			*offset = ofd->current_octet;
		}

}

int sys_stat(const char *path, struct stat *buf, int *ret) {
	*ret = vfs_stat(path, buf);
}


/*
* Waits for the task with the specified pid.
*/
int sys_waitpid(int pid, int *status)
{
int i ;
bool found;
cli;
repeat:
      found = false;

for(i = 0 ; i < MAXPID;i++){
	if (p_list[i].parent_pid == current->pid &&  p_list[i].pid == pid ) {
	found = true ;
      if (p_list[i].state == PROC_STOPPED) {
	*status = p_list[i].exit_status;
	pid = p_list[i].pid;
	return pid;
    }
}

}
sti;
if (!found) {
/* This task does not have a child with the specified pid,
or does not have any children to wait for. */
return -1;
}

/* We were awoken. Check again! */
goto repeat;
}


